<?php

namespace Drupal\Tests\field_css\FunctionalJavascript;

/**
 * Tests the CSS field widget.
 *
 * @group field_css
 */
class WidgetTest extends TestBase {

  /**
   * Tests CSS widget rendering.
   */
  public function testWidgetRendering() {
    $assert_session = $this->assertSession();

    $this->drupalLogin($this->codeUser);

    $this->drupalGet("node/add/page");

    // Verify CSS Code field is rendered.
    $assert_session->elementExists('xpath', '//textarea[@name="field_code[0][value]"]');

    // Install codemirror_editor module.
    \Drupal::service('module_installer')->install(['codemirror_editor']);

    $this->drupalGet("node/add/page");

    // Verify CSS Code field is rendered with CodeMirror editor.
    $assert_session->elementExists('xpath', '//textarea[@name="field_code[0][value]"]/following-sibling::div[contains(@class, "CodeMirror")]');

    $buttons_allowed = [
      'undo',
      'redo',
      'enlarge',
      'shrink',
    ];
    // Verify buttons are rendered.
    foreach ($buttons_allowed as $button) {
      $assert_session->elementExists('xpath', "//*[@data-cme-button='" . $button . "']");
    }
  }

  /**
   * Test widget validation.
   */
  public function testWidgetValidation() {
    $assert_session = $this->assertSession();
    $page = $this->getSession()->getPage();

    \Drupal::service('module_installer')->install([
      'block',
    ]);
    $this->drupalPlaceBlock('system_messages_block');

    $this->drupalLogin($this->codeUser);

    $this->drupalGet("node/add/page");

    $page->fillField('Title', 'A Test Page');
    $page->fillField('CSS Code', ':root p { color: red; }');
    $page->pressButton('Save');
    $assert_session->pageTextContains('The :root selector cannot be used.');

    $page->fillField('CSS Code', 'p { color: red; }');
    $page->pressButton('Save');
    $assert_session->pageTextContains('page A Test Page has been created.');
  }

  /**
   * Tests permission.
   */
  public function testWidgetPermission() {
    $assert_session = $this->assertSession();

    $this->drupalLogin($this->codeUser);

    $this->drupalGet("node/add/page");

    // Verify user has access to the CSS Code field.
    $assert_session->elementExists('xpath', '//textarea[@name="field_code[0][value]"]');

    $this->drupalLogin($this->authUser);

    $this->drupalGet("node/add/page");

    // Verify user does not have access to the CSS Code field.
    $assert_session->elementNotExists('xpath', '//textarea[@name="field_code[0][value]"]');
  }

}
